# frozen_string_literal: true

require 'telegram/bot'
require 'uri'
require 'rest-client'
require 'sanitize'

# WebProxy for a Telegram bot which handles chats and process messages
class TelegramWebProxy
  attr_accessor :bot

  def initialize(bot)
    self.bot = bot
  end

  def message_processors
    @message_processors ||= {}
  end

  def process(message)
    chat = get_chat(message)
    message_processors[chat.id] ||= MessageProcessor.new(bot, chat)
    message_processors[chat.id].process(message)
  end

  private

  def get_chat(message)
    message.respond_to?(:chat) ? message.chat : message.message.chat
  end
end

require File.dirname(__FILE__) + '/telegram_web_proxy/message_processor'
require File.dirname(__FILE__) + '/telegram_web_proxy/page_fetcher'
require File.dirname(__FILE__) + '/telegram_web_proxy/page_sanitizer'
require File.dirname(__FILE__) + '/telegram_web_proxy/keyboard_generator'
require File.dirname(__FILE__) + '/telegram_web_proxy/version'
