# frozen_string_literal: true

TelegramWebProxy::KeyboardGenerator = Struct.new(:fragments_number, :current_fragment) do
  MAX_BUTTONS = 8

  # @return [Telegram::Bot::Types::InlineKeyboardMarkup]
  def generate
    @keyboard ||=
      Telegram::Bot::Types::InlineKeyboardMarkup.new(inline_keyboard: build_keyboard)
  end

  private

  def build_keyboard
    [page_numbers.map { |i| new_button(i) }]
  end

  def page_numbers
    if fragments_number <= MAX_BUTTONS
      full_visible
    elsif current_fragment_in_the_beginning?
      beginning_visible
    elsif current_fragment_in_the_ending?
      ending_visible
    else
      middle_visible
    end
  end

  def current_fragment_in_the_beginning?
    current_fragment <= MAX_BUTTONS / 2 - 1
  end

  def current_fragment_in_the_ending?
    (fragments_number - current_fragment) <= MAX_BUTTONS / 2
  end

  def full_visible
    0..fragments_number - 1
  end

  def beginning_visible
    (0..MAX_BUTTONS - 3).to_a + [fragments_number - 1]
  end

  def middle_visible
    range = MAX_BUTTONS / 4
    [0] +
      (current_fragment - range..current_fragment + range).to_a +
      [fragments_number - 1]
  end

  def ending_visible
    [0] + (fragments_number - MAX_BUTTONS + 2..fragments_number - 1).to_a
  end

  def new_button(idx)
    Telegram::Bot::Types::InlineKeyboardButton.new(text: button_text(idx),
                                                   callback_data: "/f #{idx}")
  end

  def button_text(idx)
    if idx == current_fragment
      ">#{idx + 1}<"
    elsif idx.zero?
      '1|'
    elsif idx == fragments_number - 1
      "|#{idx + 1}"
    else
      idx + 1
    end
  end
end
